@@ -29,6 +29,7 @@ import qualified Data.ByteString.Lazy            as L
 import qualified Data.ByteString.Unsafe          as B
 import           Data.IORef                      (IORef, newIORef, readIORef,
                                                   writeIORef)
+import qualified Data.TLSSetting                 as TLS
 import           Data.Typeable
 import           Data.Word
 import           Database.MySQL.Protocol.Auth
@@ -36,13 +37,12 @@ import           Database.MySQL.Protocol.Command
 import           Database.MySQL.Protocol.Packet
 import           Network.Socket                  (HostName, PortNumber)
 import qualified Network.Socket                  as N
+import qualified Network.TLS                     as TLS
 import           System.IO.Streams               (InputStream, OutputStream)
 import qualified System.IO.Streams               as Stream
 import qualified System.IO.Streams.Binary        as Binary
 import qualified System.IO.Streams.TCP           as TCP
 import qualified System.IO.Streams.TLS           as TLS
-import qualified Data.TLSSetting                 as TLS
-import qualified Network.TLS                     as TLS
 
 --------------------------------------------------------------------------------
 
@@ -155,10 +155,10 @@ connectDetail ci@(ConnectInfo host port _ _ _ tls) =
     loopRead acc k is = do
         bs <- Stream.read is
         case bs of Nothing -> throwIO NetworkException
-                   Just bs' -> do let l = B.length bs'
+                   Just bs' -> do let l = fromIntegral (B.length bs')
                                   if l >= k
                                   then do
-                                      let (a, rest) = B.splitAt k bs'
+                                      let (a, rest) = B.splitAt (fromIntegral k) bs'
                                       unless (B.null rest) (Stream.unRead rest is)
                                       return $! L.fromChunks (reverse (a:acc))
                                   else do
@@ -186,11 +186,16 @@ command :: MySQLConn -> Command -> IO OK
 command conn@(MySQLConn is os _ _) cmd = do
     guardUnconsumed conn
     writeCommand cmd os
+    waitCommandReply is
+{-# INLINE command #-}
+
+waitCommandReply :: InputStream Packet -> IO OK
+waitCommandReply is = do
     p <- readPacket is
     if  | isERR p -> decodeFromPacket p >>= throwIO . ERRException
         | isOK  p -> decodeFromPacket p
         | otherwise -> throwIO (UnexpectedPacket p)
-{-# INLINE command #-}
+{-# INLINE waitCommandReply #-}
 
 readPacket :: InputStream Packet -> IO Packet
 readPacket is = Stream.read is >>= maybe
@@ -209,13 +214,19 @@ readPacket is = Stream.read is >>= maybe
 {-# INLINE readPacket #-}
 
 writeCommand :: Command -> OutputStream Packet -> IO ()
-writeCommand a = let bs = Binary.runPut (Binary.put a) in
-    go (fromIntegral (L.length bs)) 0 bs
+writeCommand a os = let bs = Binary.runPut (Binary.put a) in
+    go (fromIntegral (L.length bs)) 0 bs os
   where
-    go len seqN bs =
+    go len seqN bs os' = do
         if len < 16777215
-        then Stream.write (Just (Packet len seqN bs))
-        else go (len - 16777215) (seqN + 1) (L.drop 16777215 bs)
+        then Stream.write (Just (Packet len seqN bs)) os'
+        else do
+            let (bs', rest) = L.splitAt 16777215 bs
+                seqN' = seqN + 1
+                len'  = len - 16777215
+
+            Stream.write (Just (Packet 16777215 seqN bs')) os'
+            seqN' `seq` len' `seq` go len' seqN' rest os'
 {-# INLINE writeCommand #-}
 
 guardUnconsumed :: MySQLConn -> IO ()
@@ -261,6 +272,7 @@ clientCap =  CLIENT_LONG_PASSWORD
                 .|. CLIENT_IGNORE_SPACE
                 .|. CLIENT_PROTOCOL_41
                 .|. CLIENT_TRANSACTIONS
+                .|. CLIENT_MULTI_STATEMENTS
                 .|. CLIENT_SECURE_CONNECTION
 
 clientMaxPacketSize :: Word32
